
#include "Map.h"

#pragma once

class Node
{
    public:
    uint64_t hash;
    uint64_t children[4];
    bool viable[4];
    double wins[4];
    double trials[4];
    uint64_t visits;
    bool aged;

    Node() : hash(0), visits(0), aged(false)
    {
        for (int i=0; i < 4; i++)
        {
            children[i] = false;
            viable[i] = false;
            wins[i] = 1;
            trials[i] = 1;
        }
    }

    void set(Map& position);
};

struct NodeTableCell
{
    Node always;
    Node high_count;
};

class NodeTable
{
    public:
    NodeTable();
    void resizeTable(int size);
    void get(Node& result, Map& position);
    void set(const Node& node);
    void ageTable();

    private:
    NodeTableCell* table;
    int table_size;
};

class UCTSearch
{
    public:
    UCTSearch();
    int lightStep(const Map& position);
    double playout(const Map& start);
    double searchTree(Map& position);
    int search(Map& position, uint64_t start_time, int search_time);
    void postSearch();

    private:
    NodeTable* nodes;
    uint64_t steps_taken;
    uint64_t num_playouts;
    uint64_t total_depth;
    int max_depth;
    int depth;
#ifdef TRACE_TREE
    public:
    bool print_tree;
#endif
};

